% Visualization of a conformal mapping
function conformal_mapping
global d
fsize = 18;

%Original Argand diagram limits
xlimits = [-4,4];
ylimits = [-4,4];

%Number of grid lines, and points per line
N = 41; d.M=1000;

%Generate grid lines
xlines = linspace( xlimits(1),xlimits(2),N).';
ylines = linspace( ylimits(1),ylimits(2),N).';
xxlines = linspace( xlimits(1),xlimits(2),d.M).';
yylines = linspace( ylimits(1),ylimits(2),d.M).';

%Horizontal lines
x = []; y = [];
for n=1:N
    x = [x,xxlines];
    y = [y,ylines(n)*ones(d.M,1)];
end
%Vertical lines
for n=1:N
    x = [x,xlines(n)*ones(d.M,1)];
    y = [y,yylines];
end

%Transform grid
[xx,yy] = ztrans(x,y);

%Initial centre and radius of circle, and keypress increments
d.xc = -0.08; d.yc = 0.08; d.dxyc = 0.01;
d.r = sqrt( (1-d.xc)^2 + d.yc^2 ); d.dr = 0.01;

%Create circle
t = linspace(0,2*pi,d.M);
d.xcircle = d.r*cos(t) + d.xc; d.ycircle = d.r*sin(t) + d.yc;

%Transform circle
[d.xxcircle,d.yycircle] = ztrans(d.xcircle,d.ycircle);

%Plot grid and circle
figure('units','normalized','position',[0 0 1 1],'keypressfcn',@keypress);
subplot(1,2,1); set(gca,'fontsize',fsize);
plot(x,y,'b-'); axis equal; hold on;
plot( xlimits, [0,0],'k-','linewidth',2 ); plot( [0,0],ylimits,'k-','linewidth',2);
d.circle = plot( d.xcircle, d.ycircle, 'r','linewidth',2);
xlabel('x'); ylabel('y'); d.tit = title(['r=',num2str(d.r),...
    ', xc=',num2str(d.xc),', yc=',num2str(d.yc)]) ;
xlim(xlimits); ylim(ylimits);

%Plot transformed grid and circle
subplot(1,2,2); set(gca,'fontsize',fsize);
plot(xx,yy,'b-'); axis equal; hold on;
plot( xlimits, [0,0],'k-','linewidth',2 ); plot( [0,0],ylimits,'k-','linewidth',2 );
d.ccircle = plot( d.xxcircle, d.yycircle, 'r','linewidth',2  );
xlabel('xx'); ylabel('yy'); title('Transformed Argand plane');
xlim(xlimits); ylim(ylimits);

%%

%Conformal mapping (i.e. transformation of the complex plane).
function [xx,yy] = ztrans(x,y)
z = x + 1i*y;
z = z + 1./z;
xx = real(z); yy = imag(z);

%%

%Key press handler
function keypress(hObject, eventdata, handles)
global d

%Get the string of the current key press
s = get( gcf,'currentkey');

%Actions, depending on the key pressed...
if ( strcmp(s,'q') == 1 ) || ( strcmp(s,'Q') == 1 )
    %Quit GUI
    close(gcf);
elseif strcmp(s,'p') == 1
    %Print figure to a PNG file
    s = ['r=',num2str(d.r),', xc=',...
        num2str(d.xc),', yc=',num2str(d.yc)];
    s = strrep(s,'.','p');
    print( gcf, [s,'.png'],'-dpng','-r300');
elseif strcmp(s,'uparrow') == 1
    d.yc = d.yc + d.dxyc;
elseif ( strcmp(s,'downarrow') == 1 )
    d.yc = d.yc - d.dxyc;
elseif strcmp(s,'rightarrow') == 1
    d.xc = d.xc + d.dxyc;
elseif ( strcmp(s,'leftarrow') == 1 )
    d.xc = d.xc - d.dxyc;
elseif ( strcmp(s,'r') == 1 )
    d.r = d.r + d.dr; if d.r<0; d.r = 0; end;
elseif ( strcmp(s,'e') == 1 )
    d.r = d.r - d.dr;
elseif ( strcmp(s,'k') == 1 )
    %Change circle radius to satisfy Kutta condition
    d.r = sqrt( (1-d.xc)^2 + d.yc^2 );
end

%Update circle and transformed circle
t = linspace(0,2*pi,d.M);
d.xcircle = d.r*cos(t) + d.xc; d.ycircle = d.r*sin(t) + d.yc;
[d.xxcircle,d.yycircle] = ztrans(d.xcircle,d.ycircle);
set( d.circle,'xdata',d.xcircle,'ydata',d.ycircle );
set( d.ccircle,'xdata',d.xxcircle,'ydata',d.yycircle );
set( d.tit,'string',['r=',num2str(d.r),', xc=',...
    num2str(d.xc),', yc=',num2str(d.yc)] );

%End of code

